@@ -59,7 +59,6 @@ Mainmenu::Mainmenu(){
         connect(proc,SIGNAL(stateChanged(QProcess::ProcessState)),this,SLOT(updateProgressDialog()));
         connect(proc,SIGNAL(finished(int)),this,SLOT(resetProgressDialog()));
 
-
     }
 
     void Mainmenu::putEntriesToLayout(){
@@ -82,8 +81,16 @@ Mainmenu::Mainmenu(){
 
             QComboBox *comboBox = new QComboBox;
 
-            catList(comboBox);
-            comboBox->setCurrentIndex(Entry::listCatoriseAvailMens.indexOf(entry->getActualCategory()));
+            catList(comboBox, 1);
+            QString actCatName = entry->getActualCategory();
+
+            if (!oldEditName.isEmpty()) {
+                for (int i = 0; i < oldEditName.size(); i++) {
+                    actCatName.replace(QRegExp("^"+oldEditName.at(i)), newEditName.at(i));
+                }
+            }
+
+            comboBox->setCurrentIndex(Entry::listCatoriseAvailMens.indexOf( actCatName ));
 
             connect(comboBox,SIGNAL(currentIndexChanged(int)),entry,SLOT(changedCategory(int)));
 
@@ -101,7 +108,11 @@ Mainmenu::Mainmenu(){
 
             Entry *entry = new Entry();
             entry->setDesktopName(tmp.section(':',0,0).trimmed());
-            entry->setActualCategory(tmp.section(":",1,1).trimmed());
+            QString actCat = tmp.section(":",1,1).trimmed();
+//            if (actCat == "_root") {
+//                actCat = "No Category";
+//            }
+            entry->setActualCategory(actCat);
 
             readDesktopFileInformation(entry);
 
@@ -140,10 +151,10 @@ Mainmenu::Mainmenu(){
             QTextStream in(&file);
             while(!in.atEnd()){
                 line = in.readLine();
-                if(!name && line.contains("name=",Qt::CaseInsensitive)){
+                if(!name && (line.contains("name=",Qt::CaseInsensitive) || line.contains("name =",Qt::CaseInsensitive))){
                     entry->setName( line.section("=",1,1).trimmed() );
                     name = !name;
-                }else if(!icon && line.contains("icon=",Qt::CaseInsensitive)){
+                }else if(!icon && (line.contains("icon=",Qt::CaseInsensitive) || line.contains("icon =",Qt::CaseInsensitive))){
                     entry->setIconPath( getIconPath( line.section("=",1,1).trimmed() ) );
                     icon = !icon;
                 }else if(!textDomain && line.contains("x-text-domain=",Qt::CaseInsensitive)){
@@ -213,7 +224,7 @@ Mainmenu::Mainmenu(){
     void Mainmenu::showAbout()
     {
         QMessageBox::about(this, tr("About Catorise PLUS"),
-                tr("<p><b>Catorise PLUS</b> is a extended version from catorise with a graphical user interface.<br />"
+                tr("<p><b>Catorise PLUS</b> is a extended version of catorise with a graphical user interface.<br />"
                    "It is originally based on catorise by <a href=\"http://www.bleb.org/\">Andrew Flegg</a> and Catorise GUI by Recep Karadas (<a href=\"http://www.reckdesign.de\" >Reckdesign</a>)"
                    "</p>"));
     }
@@ -223,7 +234,11 @@ Mainmenu::Mainmenu(){
         QString menu;
 
         for(int i=0;i<listEntries->size();i++){
-            menu += QString(listEntries->at(i)->getDesktopName() + ": " + listEntries->at(i)->getActualCategory());
+            QString actCat = listEntries->at(i)->getActualCategory();
+//            if (actCat == "No Category") {
+//                actCat = "_root";
+//            }
+            menu += QString(listEntries->at(i)->getDesktopName() + ": " + actCat);
             if(i!=listLabels->size()-1){
                 menu +="\n";
             }
@@ -254,23 +269,24 @@ Mainmenu::Mainmenu(){
     void Mainmenu::addCategory() {
         QDialog *addCatDialog = new QDialog(this);
         QGridLayout *addCatLayout = new QGridLayout(addCatDialog);
+        addCatCBox = new QComboBox;
 
         QLabel *addCatLabelName = new QLabel("Category Name:"); // Category Name (Label)
+        QLabel *addCatLabelSub = new QLabel("Subcategory:"); // Subcategory (Label)
         QLabel *addCatLabelIcon = new QLabel("Category Icon:"); // Category Icon (Label)
-        QLabel *addCatLabelIconSelect = new QLabel("or select a file:"); // Category Icon (Select, Label)
 
         addCatEditName = new QLineEdit(); // Category Name (Edit Box)
-        addCatEditIcon = new QLineEdit("app_install_"); // Category Icon (Edit Box)
+        catList(addCatCBox, 1); // Subcategory
         addCatSelectIcon = new QPushButton("Choose a icon file"); // Category Icon (Select)
 
         QPushButton *addCatButtonOK = new QPushButton(tr("&OK")); // OK Button
         QPushButton *addCatButtonAbort = new QPushButton(tr("&Abort")); // Abort Button
 
         addCatLayout->addWidget(addCatLabelName,0,0,0);
-        addCatLayout->addWidget(addCatLabelIcon,1,0,0);
-        addCatLayout->addWidget(addCatLabelIconSelect,2,0,0);
+        addCatLayout->addWidget(addCatLabelSub,1,0,0);
+        addCatLayout->addWidget(addCatLabelIcon,2,0,0);
         addCatLayout->addWidget(addCatEditName,0,1,0); // Category Name
-        addCatLayout->addWidget(addCatEditIcon,1,1,0); // Category Icon
+        addCatLayout->addWidget(addCatCBox,1,1,0); // Subcategory
         addCatLayout->addWidget(addCatSelectIcon,2,1,0); // Category Icon (Select)
 
         addCatLayout->addWidget(addCatButtonOK,0,2,0); // OK Button
@@ -285,10 +301,35 @@ Mainmenu::Mainmenu(){
     }
 
     void Mainmenu::selectCatIcon() {
-        catIconPath = QFileDialog::getOpenFileName(this, "Choose a file", "/home/user/MyDocs/", "Images (*.png)");
+        IconCatDialog = new QDialog(this);
+        QGridLayout *IconCatLayout = new QGridLayout(IconCatDialog);
+
+        QPushButton *IconCatButton1 = new QPushButton(tr("Select from MyDocs"));
+        QPushButton *IconCatButton2 = new QPushButton(tr("Select from System-Icon-Folder"));
+
+        IconCatLayout->addWidget(IconCatButton1,0,0,0);
+        IconCatLayout->addWidget(IconCatButton2,1,0,0);
+
+        connect(IconCatButton1,SIGNAL(clicked()),this,SLOT( selectIconFromMyDocs() ));
+        connect(IconCatButton2,SIGNAL(clicked()),this,SLOT( selectIconFromSystemDir() ));
+
+        IconCatDialog->exec();
+    }
+
+    void Mainmenu::selectIconFromMyDocs() {
+        selectIcon("/home/user/MyDocs/");
+    }
+
+    void Mainmenu::selectIconFromSystemDir() {
+        selectIcon("/usr/share/icons/hicolor/64x64/hildon/");
+    }
+
+    void Mainmenu::selectIcon(QString dir) {
+        catIconPath = QFileDialog::getOpenFileName(this, "Choose a file", dir, "Images (*.png)");
         if (catIconPath != NULL) {
             addCatSelectIcon->setIcon( QIcon(catIconPath) );
             addCatSelectIcon->setText( catIconPath.mid( catIconPath.lastIndexOf("/")+1 ) );
+            IconCatDialog->done(1);
         }
     }
 
@@ -300,55 +341,77 @@ Mainmenu::Mainmenu(){
         // Open file
         QString createCatFileName = addCatEditName->text();
         createCatFileName.replace("ai_category_", "");
-        if (createCatFileName != NULL) {
-            QFile file("/opt/catorise/share/"+createCatFileName+".directory");
-            if (file.exists() != 1) {
-                if (file.open(QIODevice::WriteOnly | QIODevice::Text)) {
-                    // Icon file
-                    QString catIcon;
-                    if (catIconPath != NULL) {
-                        QString catIconBasename = catIconPath;
-                        catIconBasename = catIconBasename.mid( catIconBasename.lastIndexOf("/")+1 );
-                        catIconBasename.chop(4);
-                        catIconBasename = "catorise_cat_"+catIconBasename;
-                        QProcess *exeApp = new QProcess(this);
-                        exeApp->start("/bin/busybox sh -c \"echo cp '"+catIconPath+"' '/opt/usr/share/icons/hicolor/64x64/hildon/"+catIconBasename+".png'|root\""); // Copy file
-                        catIcon = catIconBasename;
+        if (createCatFileName != NULL && createCatFileName != QString("No Category")) {
+            // Subcategory
+            int DirError = 0;
+            QString dirName;
+            if (addCatCBox->currentIndex() > 0) {
+                // create sub-dir
+                dirName = Entry::listCatoriseAvailMens.at( addCatCBox->currentIndex() );
+                QDir dir;
+                if (dir.exists("/opt/catorise/share/"+dirName) == 0) {
+                    if (dir.mkdir("/opt/catorise/share/"+dirName) != 0) {
+                        // Dir created
+                        createCatFileName = dirName + "/" + createCatFileName;
                     }
                     else {
-                        catIcon = addCatEditIcon->text();
+                        // error
+                        DirError = 1;
                     }
-
-                    // Create file with info
-                    QString CatInfo = QString("[Desktop Entry]\n" \
-                                              "Type=Directory\n" \
-                                              "Name="+addCatEditName->text()+"\n" \
-                                              "Icon="+catIcon+"\n" \
-                                              "X-Text-Domain=hildon-application-manager");
-                    QTextStream saveCat(&file);
-                    saveCat << CatInfo;
-                    // Close file
-                    file.close();
-
-                    if (noMessage == 0) {
-                        // set message
-                        showInformationBox("Category \""+ ( QString::fromLocal8Bit(gettext(addCatEditName->text().toUtf8()) ) ).trimmed() +"\" was added.\nPlease click \"Save\" to reload the menu.", 1);
-                    }
-
-                    // Generate new list
-                    Entry *entry;
-                    entry->CreateList();
-                    putEntriesToLayout();
-    
-                   // Start Catorise to refresh the menu
-//                   proc->start("/bin/busybox sh -c \"echo /opt/catorise/catorise|root\"");
                 }
                 else {
-                    showInformationBox("File /opt/catorise/share/"+createCatFileName+".directory can't be open.");
+                    createCatFileName = dirName + "/" + createCatFileName;
+                }
+            }
+
+            if (DirError == 0) {
+                QFile file("/opt/catorise/share/"+createCatFileName+".directory");
+                if (file.exists() != 1) {
+                    if (file.open(QIODevice::WriteOnly | QIODevice::Text)) {
+                        // Icon file
+                        if (catIconPath != NULL) {
+                            QString catIconBasename = catIconPath;
+                            catIconBasename = catIconBasename.mid( catIconBasename.lastIndexOf("/")+1 );
+                            catIconBasename.chop(4);
+                            catIconBasename = "catorise_cat_"+catIconBasename;
+                            QProcess *exeApp = new QProcess(this);
+                            exeApp->start("/bin/busybox sh -c \"echo cp '"+catIconPath+"' '/opt/usr/share/icons/hicolor/64x64/hildon/"+catIconBasename+".png'|root\""); // Copy file
+                            catIcon = catIconBasename;
+                        }
+                        else {
+                            catIcon = "";
+                        }
+
+                        // Create file with info
+                        QString CatInfo = QString("[Desktop Entry]\n" \
+                                                  "Type=Directory\n" \
+                                                  "Name="+addCatEditName->text()+"\n" \
+                                                  "Icon="+catIcon+"\n" \
+                                                  "X-Text-Domain=hildon-application-manager");
+                        QTextStream saveCat(&file);
+                        saveCat << CatInfo;
+                        // Close file
+                        file.close();
+
+                        if (noMessage == 0) {
+                            // set message
+                            showInformationBox("Category \""+ ( QString::fromLocal8Bit(gettext(addCatEditName->text().toUtf8()) ) ).trimmed() +"\" was added.\nPlease click \"Save\" to reload the menu.", 1);
+                            // Generate new list
+                            Entry *entry;
+                            entry->CreateList();
+                            putEntriesToLayout();
+                        }
+                    }
+                    else {
+                        showInformationBox("File /opt/catorise/share/"+createCatFileName+".directory can't be open.");
+                    }
+                }
+                else {
+                    showInformationBox("Category \""+ ( QString::fromLocal8Bit(gettext(addCatEditName->text().toUtf8()) ) ).trimmed() +"\" exists.");
                 }
             }
             else {
-                showInformationBox("Category \""+ ( QString::fromLocal8Bit(gettext(addCatEditName->text().toUtf8()) ) ).trimmed() +"\" exists.");
+                showInformationBox("File /opt/catorise/share/"+dirName+" can't be make.");
             }
         }
         else {
@@ -366,7 +429,7 @@ Mainmenu::Mainmenu(){
         delCatLabel = new QLabel("Select a category you want to delete."); // Label
 
         // List categories
-        catList(delCatCBox);
+        catList(delCatCBox, 0);
 
         delCatLayout->addWidget(delCatLabel,0,0,1,0,0); // Label
         delCatLayout->addWidget(delCatCBox,1,0,1,0,0); // Select Box
@@ -383,21 +446,17 @@ Mainmenu::Mainmenu(){
     }
 
     void Mainmenu::deleteCategory(int noMessage) {
-        QString delCatName = Entry::listCatoriseAvailMens.at( delCatCBox->currentIndex() );
+        QString delCatName = Entry::listCatoriseAvailMens.at( delCatCBox->currentIndex()+1 );
         QFile *delFileName = new QFile();
         if (delFileName->remove("/opt/catorise/share/"+delCatName+".directory") == 1) {
            if (noMessage == 0) {
                // set message
                showInformationBox("Category \""+delCatCBox->currentText()+"\" was deleted.\nPlease click \"Save\" to reload the menu.", 1);
+               // Generate new list
+               Entry *entry;
+               entry->CreateList();
+               putEntriesToLayout();
            }
-
-           // Generate new list
-           Entry *entry;
-           entry->CreateList();
-           putEntriesToLayout();
-
-           // Start Catorise to refresh the menu
-//           proc->start("/bin/busybox sh -c \"echo /opt/catorise/catorise|root\"");
         }
         else {
            // Cannot delete
@@ -411,17 +470,29 @@ Mainmenu::Mainmenu(){
         
         QLabel *editCatLabel = new QLabel("Select a categorie that you want edit.");
         QLabel *editCatLabelName = new QLabel("Category Name:"); // Category Name (Label)
+        QLabel *editCatLabelSub = new QLabel("Subcategory:"); // Subcategory
         QLabel *editCatLabelIcon = new QLabel("Category Icon:"); // Category Icon (Label)
-        QLabel *editCatLabelIconSelect = new QLabel("or select a file:"); // Category Icon (Select, Label)
 
         // List categories
         delCatCBox = new QComboBox();
-        catList(delCatCBox);
+        addCatCBox = new QComboBox();
+        catList(delCatCBox, 0);
+        catList(addCatCBox, 1);
 
-        getNameAndIcon( Entry::listCatoriseAvailMens.at( delCatCBox->currentIndex() ) ); // Get current name and icon
+        getSubCat( Entry::listCatoriseAvailMens.at( delCatCBox->currentIndex()+1 ) );
+        getSubAndName( catName );
         addCatEditName = new QLineEdit( catName ); // Category Name (Edit Box)
-        addCatEditIcon = new QLineEdit( catIcon ); // Category Icon (Edit Box)
-        addCatSelectIcon = new QPushButton("Choose a icon file"); // Category Icon (Select)
+        addCatSelectIcon = new QPushButton(); // Category Icon (Select)
+
+        // Get Icon
+        if (catIcon != NULL) {
+            addCatSelectIcon->setIcon(QIcon("/usr/share/icons/hicolor/64x64/hildon/"+catIcon+".png"));
+            addCatSelectIcon->setText(catIcon);
+        }
+        else {
+            addCatSelectIcon->setIcon(QIcon());
+            addCatSelectIcon->setText("Choose a icon file");
+        }
 
         QPushButton *editCatButtonOK = new QPushButton(tr("&OK")); // OK Button
         QPushButton *editCatButtonAbort = new QPushButton(tr("&Abort")); // Abort Button
@@ -429,10 +500,10 @@ Mainmenu::Mainmenu(){
         editCatLayout->addWidget(editCatLabel,0,0,1,2,0);
         editCatLayout->addWidget(delCatCBox,1,0,1,3,0);
         editCatLayout->addWidget(editCatLabelName,2,0,0);
-        editCatLayout->addWidget(editCatLabelIcon,3,0,0);
-        editCatLayout->addWidget(editCatLabelIconSelect,4,0,0);
+        editCatLayout->addWidget(editCatLabelSub,3,0,0);
+        editCatLayout->addWidget(editCatLabelIcon,4,0,0);
         editCatLayout->addWidget(addCatEditName,2,1,1,3,0); // Category Name
-        editCatLayout->addWidget(addCatEditIcon,3,1,1,3,0); // Category Icon
+        editCatLayout->addWidget(addCatCBox,3,1,1,3,0); // Subcategory
         editCatLayout->addWidget(addCatSelectIcon,4,1,1,3,0); // Category Icon (Select)
 
         editCatLayout->addWidget(editCatButtonOK,0,3,0); // OK Button
@@ -453,7 +524,7 @@ Mainmenu::Mainmenu(){
             QTextStream fileContent(&file);
             while (!fileContent.atEnd()) {
                 QRegExp lineFindName("Name=(.+)");
-                QRegExp lineFindIcon("Icon=(.+)");
+                QRegExp lineFindIcon("Icon=(.*)");
                 QString fileC = fileContent.readLine();
                 lineFindName.indexIn( fileC );
                 lineFindIcon.indexIn( fileC );
@@ -461,7 +532,7 @@ Mainmenu::Mainmenu(){
                 if (lineFindName.cap(1) != NULL) {
                     catName = lineFindName.cap(1);
                 }
-                else if (lineFindIcon.cap(1) != NULL) {
+                else if (lineFindIcon.cap(1) != NULL || lineFindIcon.cap(0) != NULL) {
                     catIcon = lineFindIcon.cap(1);
                 }
             }
@@ -469,30 +540,115 @@ Mainmenu::Mainmenu(){
         }
     }
 
+    void Mainmenu::getSubCat(QString fileName) {
+        if (fileName.contains("/")) {
+            QStringList subCats = fileName.split("/");
+            QString newCatName = NULL;
+            QString subCatDir = NULL;
+            for (int i = 0; i < subCats.size(); i++) {
+                if (subCatDir == NULL) {
+                    subCatDir = subCats.at(i);
+                }
+                else {
+                    subCatDir = subCatDir+"/"+subCats.at(i);
+                }
+                getNameAndIcon(subCatDir);
+                newCatName = newCatName+"/"+( QString::fromLocal8Bit(gettext(catName.toUtf8()) ) ).trimmed();
+            }
+            catName = newCatName.remove(0,1);
+        }
+        else {
+            getNameAndIcon(fileName);
+        }
+    }
+
+    void Mainmenu::getSubAndName(QString name) {
+        if (name.contains("/")) {
+            int SlashPos = name.lastIndexOf("/");
+            QString catIconTmp = catIcon;
+            QString catNameTmp = name.mid(SlashPos+1);
+            QString subCatName = name.mid(0, SlashPos);
+            // set correct subcategory
+            for (int i = 0; i < Entry::listCatoriseAvailMens.length(); i++) {
+                getSubCat(Entry::listCatoriseAvailMens.at(i));
+                if ( ( QString::fromLocal8Bit(gettext(catName.toUtf8()) ) ).trimmed() == subCatName) {
+                    addCatCBox->setCurrentIndex(i);
+                    break;
+                }
+            }
+            catName = catNameTmp;
+            catIcon = catIconTmp;
+        }
+        else {
+            catName = name;
+            addCatCBox->setCurrentIndex(0);
+        }
+    }
+
     void Mainmenu::selectCatEdit() {
-        getNameAndIcon( Entry::listCatoriseAvailMens.at( delCatCBox->currentIndex() ) );
+        getSubCat( Entry::listCatoriseAvailMens.at( delCatCBox->currentIndex()+1 ) );
+        getSubAndName( catName );
         addCatEditName->setText( catName );
-        addCatEditIcon->setText( catIcon );
+
+        // Get Icon
+        if (catIcon != NULL) {
+            addCatSelectIcon->setIcon(QIcon("/usr/share/icons/hicolor/64x64/hildon/"+catIcon+".png"));
+            addCatSelectIcon->setText(catIcon);
+        }
+        else {
+            addCatSelectIcon->setIcon(QIcon());
+            addCatSelectIcon->setText("Choose a icon file");
+        }
     }
 
     void Mainmenu::editCat() {
         deleteCategory(1);
         createCategory(1);
 
+        // rename dir
+        QDir dir;
+        if (dir.exists("/opt/catorise/share/"+Entry::listCatoriseAvailMens.at( delCatCBox->currentIndex()+1 ) )) {
+            dir.rename("/opt/catorise/share/"+Entry::listCatoriseAvailMens.at( delCatCBox->currentIndex()+1 ), "/opt/catorise/share/"+Entry::listCatoriseAvailMens.at( addCatCBox->currentIndex() ) +"/"+ addCatEditName->text() );
+        }
+
+        // set old and new name from category
+        oldEditName << Entry::listCatoriseAvailMens.at( delCatCBox->currentIndex()+1 );
+        newEditName << Entry::listCatoriseAvailMens.at( addCatCBox->currentIndex() ) +"/"+ addCatEditName->text();
+
         // set message
         showInformationBox("Category \""+ delCatCBox->currentText() +"\" was edited.\nPlease click \"Save\" to reload the menu.", 1);
+
+        // Generate new list
+        Entry *entry;
+        entry->CreateList();
+        putEntriesToLayout();
     }
 
-    void Mainmenu::catList(QComboBox *&cbox) {
+    void Mainmenu::catList(QComboBox *&cbox, int viewNoCat) {
         for(int k=0;k<Entry::listCatoriseAvailMens.length();k++){
+            if (viewNoCat == 0 && k == 0) {
+                continue;
+            }
+            else if (viewNoCat == 1 && k == 0) {
+                cbox->insertItem(k, Entry::listCatoriseAvailMens.at(k));
+            }
+            else {
+                int j;
+                if (viewNoCat == 0) {
+                    j = k-1;
+                }
+                else {
+                   j = k;
+                }
                 // get filename and iconname
-                getNameAndIcon(Entry::listCatoriseAvailMens.at(k));
+                getSubCat( Entry::listCatoriseAvailMens.at(k) );
 
                 if(catName.operator ==("ai_category_ovi")){
-                    cbox->insertItem(k, QIcon("/opt/usr/share/icons/hicolor/64x64/hildon/"+catIcon+".png") ,"Ovi");
+                    cbox->insertItem(j, QIcon("/opt/usr/share/icons/hicolor/64x64/hildon/"+catIcon+".png") ,"Ovi");
                 }else{
-                    cbox->insertItem(k, QIcon("/opt/usr/share/icons/hicolor/64x64/hildon/"+catIcon+".png") , ( QString::fromLocal8Bit(gettext(catName.toUtf8()) ) ).trimmed() );
+                    cbox->insertItem(j, QIcon("/opt/usr/share/icons/hicolor/64x64/hildon/"+catIcon+".png") , ( QString::fromLocal8Bit(gettext(catName.toUtf8()) ) ).trimmed() );
                 }
+            }
         }
     }
 
